from ..view import View, Data, Action, pimp_my_boolean
from .schema import AddDomainSchema, AddSubDomainSchema, EditDomainSchema
from gocept.directory2.domain import Domain
from pyramid.view import view_config
from deform.widget import TextAreaWidget
import colander
import gocept.directory2.admin.form


class MainTabs(object):

    @property
    def tabs(self):
        return [
            Action('Domains', '', 'domain_list'),
            Action('Users', '', 'domain_user_list')
        ]


class ListDomainsBase(MainTabs, View):
    """List known domains."""
    title = 'Domains'
    actions = [Action('Add domain', 'icon-plus', 'domain_add')]

    def filter(self, query):
        return query.filter(Domain.subdomain_of == None)

    @property
    def domains(self):
        for x in self.filter(Domain.query().order_by(Domain._name)):
            yield Data(name=x.name,
                       user=x.user.name,
                       ip=x.ip,
                       view_url=self.request.route_url('domain_view', id=x.id),
                       user_url=self.request.route_url(
                           'domain_user_view', id=x.user.id))


@view_config(route_name='domain_list',
             renderer='templates/domain_list.pt')
class ListDomains(ListDomainsBase):
    """List known domains."""


class DomainTabs(object):

    type_ = 'domain'

    @property
    def tabs(self):
        return [
            Action('Info', '', '{}_view'.format(self.type_),
                   id=self.context.id),
            Action('Subdomains', '', 'subdomain_list', id=self.context.id)
        ]

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit', '{}_edit'.format(self.type_),
                     id=self.context.id)
        yield Action('Delete', 'icon-remove', 'domain_confirm_delete',
                     id=self.context.id)

    def pimp_my_boolean(self, value):
        return pimp_my_boolean(value)


class SubDomainTabs(DomainTabs):

    type_ = 'subdomain'

    @property
    def tabs(self):
        return super(SubDomainTabs, self).tabs[:-1]

    @property
    def actions(self):
        yield Action('Back to domain', 'icon-arrow-left', 'domain_view',
                     id=self.context.subdomain_of.id)
        for action in super(SubDomainTabs, self).actions:
            yield action


class ViewBase(View):

    schema = NotImplemented  # schema with the fields which shall be displayed

    def get_value(self, context, node):
        """Retrieve the most meaningful value for node from context."""
        value = getattr(context, node.name, '')
        if isinstance(node.typ, colander.Boolean):
            return pimp_my_boolean(value)
        if isinstance(node.widget, TextAreaWidget):
            return '<pre>{}</pre>'.format(value)
        return value

    @property
    def display_fields(self):
        """Return title and value for all nodes on the domain add schema."""
        for node in self.schema():
            yield {'title': node.title,
                   'value': self.get_value(self.context, node)}


class ViewDomainBase(ViewBase):

    schema = AddDomainSchema

    def __call__(self):
        self.title = self.context.name
        self.subtitle = self.context.user.name
        self.description = self.context.ip
        return super(ViewDomainBase, self).__call__()

    def get_value(self, context, node):
        """Retrieve the most meaningful value for node from context."""
        value = super(ViewDomainBase, self).get_value(context, node)
        if isinstance(node.typ, gocept.directory2.admin.schema.Relation):
            return '<a href="{}">{}</a>'.format(
                self.request.route_url('domain_user_view', id=value.id),
                value.name)
        return value


@view_config(route_name='domain_view',
             renderer='templates/view.pt')
class ViewDomain(DomainTabs, ViewDomainBase):
    """View a single domain."""


@view_config(route_name='subdomain_view',
             renderer='templates/view.pt')
class SubDomainView(SubDomainTabs, ViewDomainBase):
    """Display one subdomain."""


@view_config(route_name='subdomain_list',
             renderer='templates/subdomain_list.pt')
class SubDomainList(DomainTabs, View):
    """List subdomains for current domain."""

    def __call__(self):
        self.title = 'Subdomains of {}'.format(self.context.name)
        return super(SubDomainList, self).__call__()

    @property
    def actions(self):
        return [Action('Add subdomain', 'icon-plus', 'subdomain_add',
                       id=self.context.id)]

    @property
    def subdomains(self):
        for x in Domain.query().order_by(Domain._name).filter(
                Domain.subdomain_of == self.context):
            yield Data(name=x.name,
                       web=pimp_my_boolean(x.httpd_conf),
                       mail=pimp_my_boolean(x.sendmail_cw),
                       dns=pimp_my_boolean(x.named_conf),
                       path=x.rel_path,
                       ip=x.ip,
                       view_url=self.request.route_url(
                           'subdomain_view', id=x.id))


@view_config(route_name='domain_add')
class AddDomain(gocept.directory2.admin.form.Add):
    """Add a new domain."""

    title = 'Add domain'
    schema = AddDomainSchema()
    factory = Domain
    next_route = 'domain_list'


@view_config(route_name='subdomain_add')
class AddSubDomain(gocept.directory2.admin.form.Add):
    """Add a new subdomain."""

    title = 'Add subdomain'
    schema = AddSubDomainSchema()
    factory = Domain
    next_route = 'subdomain_list'

    @property
    def next_route_kw(self):
        return {'id': self.context.id}

    def after_add(self, obj):
        obj.subdomain_of = self.context
        return obj


@view_config(route_name='domain_edit')
class EditDomain(DomainTabs, gocept.directory2.admin.form.Edit):

    title = 'Edit domain'
    schema = EditDomainSchema()
    next_route = 'domain_view'


@view_config(route_name='subdomain_edit')
class EditSubDomain(SubDomainTabs, gocept.directory2.admin.form.Edit):

    title = 'Edit subdomain'
    schema = EditDomainSchema()
    next_route = 'subdomain_view'


@view_config(route_name='domain_confirm_delete',
             renderer='../templates/confirm-delete.pt')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of domain."""

    delete_route = 'domain_delete'

    @property
    def subject(self):
        return 'domain "%s"' % self.context.name


@view_config(route_name='domain_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete domain."""

    @property
    def next_route(self):
        if not self.context.subdomain_of:
            return 'domain_list'
        return 'subdomain_list'

    @property
    def next_route_kw(self):
        if not self.context.subdomain_of:
            return {}
        return {'id': self.context.subdomain_of.id}

    def __call__(self):
        """Make sure subdomains are also deleted if deleting a domain."""
        if self.context.subdomain_of is None:
            [d.delete() for d in Domain.query().filter_by(
                subdomain_of=self.context).all()]
        return super(Delete, self).__call__()
