from .schema import ReadOnlyNode
from .view import View
from datetime import datetime
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_defaults
import colander
import deform.exception
import pyramid.response
import pyramid_deform


class Form(View, pyramid_deform.FormView):
    """Form without a template."""

    def __call__(self):
        result = pyramid_deform.FormView.__call__(self)
        if isinstance(result, dict):
            # rendered form
            return pyramid.response.Response(result['form'])
        return result

    def form_class(self, *args, **kw):
        self.deform_form = super(Form, self).form_class(*args, **kw)
        return self.deform_form

    def apply_appstruct(self, obj):
        # XXX design smell: it would be cleaner to pass in the appstruct here,
        # instead of requiring the caller to put it onto self.appstruct
        # beforehand. However, we need the appstruct in
        # _convert_to_validation_error(), and we want to be able use *that*
        # e.g. in after_add() -- and since passing the appstruct along all
        # those methods is rather unwieldy, we go with the self.appstruct,
        # even though it's not as clean.
        for node in self.schema:
            if isinstance(node, ReadOnlyNode):
                continue
            try:
                setattr(obj, node.name, self.appstruct[node.name])
            except AssertionError as e:
                self._convert_to_validation_error(e, node)

    def _convert_to_validation_error(self, exception, node):
        error = colander.Invalid(
            node, str(exception), self.appstruct[node.name])
        field = self.deform_form[node.name]
        field.widget.handle_error(field, error)
        raise deform.exception.ValidationFailure(
            self.deform_form, self.schema.serialize(self.appstruct),
            field.error)


class Add(Form):
    """Base add class."""

    title = NotImplemented  # title to be displayed in form
    schema = NotImplemented  # colander schema
    factory = NotImplemented  # factory to create the object on add
    next_route = NotImplemented  # redirect to this route after add
    next_route_kw = {}

    buttons = ('add',)
    form_options = (('formid', 'pyramid-deform'),
                    ('method', 'GET'))

    def after_add(self, obj):
        # Tweak object before persisting it.
        return obj

    def add_success(self, appstruct):
        self.appstruct = appstruct
        obj = self.factory()
        self.apply_appstruct(obj)
        self.after_add(obj)
        obj.persist()
        self.request.session.flash('%s added successfully.' % obj, 'success')
        return HTTPFound(location=self.request.route_url(
            self.next_route, **self.next_route_kw))


class Edit(Form):
    """Base edit class."""

    schema = NotImplemented  # colander schema
    next_route = NotImplemented  # redirect to this route after save

    buttons = ('save',)
    form_options = (('formid', 'pyramid-deform'),
                    ('method', 'GET'))

    def save_success(self, appstruct):
        self.appstruct = appstruct
        self.apply_appstruct(self.context)
        self.request.session.flash('Updated on %s' % datetime.now(), 'success')
        # XXX Maybe find better pattern to determine next URL:
        return HTTPFound(location=self.request.route_url(
            self.next_route, id=self.context.id))

    def appstruct(self):
        return {node.name: getattr(self.context, node.name)
                for node in self.schema}


@view_defaults(renderer='templates/confirm-delete.pt')
class ConfirmDelete(View):
    """Base class to confirm deletion."""

    subject = None  # info about the object for delete rendered in headline
    delete_route = NotImplemented  # route to actually delete

    @property
    def delete_url(self):
        return self.request.route_url(
            self.delete_route, id=self.context.id)


class Delete(View):
    """Base class to actually delete an object."""

    next_route = NotImplemented
    next_route_kw = {}

    def __call__(self):
        self.context.delete()
        return HTTPFound(location=self.request.route_url(
            self.next_route, **self.next_route_kw))
