from ..location import Location
from ..network import Network
from .schema import is_ascii, Relation
from .view import View, Data, Action
from colander import SchemaNode, String
from deform.widget import TextAreaWidget
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='location_list',
             renderer='templates/location/list.pt')
class ListLocations(View):
    """List known locations."""
    title = 'Locations'
    actions = [Action('Add location', 'icon-plus', 'location_add')]

    @property
    def locations(self):
        for x in Location.query():
            yield Data(title=x.name,
                       url=self.request.route_url('location_view', id=x.id))


class LocationViews(object):

    @property
    def title(self):
        return self.context.name

    @property
    def actions(self):
        yield Action('Notify all contacts', 'icon-envelope', 'location_mail',
                     id=self.context.id)
        yield Action(
            'Replace network', 'icon-resize-small', 'location_replacenetwork',
            id=self.context.id)
        yield Action('Edit', 'icon-edit', 'location_edit', id=self.context.id)
        yield Action('Delete', 'icon-remove', 'location_confirm_delete',
                     id=self.context.id)

    @property
    def tabs(self):
        return [
            Action('Info', '', 'location_view', id=self.context.id),
            Action('Networks', '', 'location_networks', id=self.context.id),
            Action('Racks', '', 'location_racks', id=self.context.id),
        ]


@view_config(route_name='location_view',
             renderer='templates/location/view.pt')
class ViewLocation(LocationViews, View):
    pass


@view_config(route_name='location_racks',
             renderer='templates/location/racks.pt')
class ViewRacks(LocationViews, View):
    pass


@view_config(route_name='location_networks',
             renderer='templates/location/networks.pt')
class ViewNetworks(LocationViews, View):
    pass


class Schema(colander.Schema):
    """Schema to add a location."""

    name = SchemaNode(String())
    abbreviation = SchemaNode(
        String(), title='Short name / Identifier', validator=is_ascii)


@view_config(route_name='location_add')
class Add(gocept.directory2.admin.form.Add):
    """Add a new location."""

    title = 'Add location'
    schema = Schema()
    factory = Location
    next_route = 'location_list'


@view_config(route_name='location_edit')
class Edit(gocept.directory2.admin.form.Edit):

    title = 'Edit location'
    schema = Schema()
    next_route = 'location_view'


@view_config(route_name='location_confirm_delete')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of location."""

    delete_route = 'location_delete'

    @property
    def subject(self):
        return 'location "%s"' % self.context.name


@view_config(route_name='location_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete location."""

    next_route = 'location_list'


class MailSchema(colander.Schema):
    """Schema to add a location."""

    subject = SchemaNode(String())
    message = SchemaNode(
        String(),
        description='Do not include an address, closing, or footer.',
        widget=TextAreaWidget())


@view_config(route_name='location_mail')
class MailContacts(gocept.directory2.admin.form.Form):
    """Send a mail to all technical contacts in the location."""

    buttons = ('send',)
    schema = MailSchema()
    title = 'E-mail all technical contacts'
    next_route = 'location_view'

    def send_success(self, appstruct):
        self.context.mail_contacts(appstruct['subject'], appstruct['message'])
        self.request.session.flash('Successfully sent message.', 'success')
        # XXX Maybe find better pattern to determine next URL:
        return HTTPFound(location=self.request.route_url(
            self.next_route, id=self.context.id))


class ReplaceSchema(colander.Schema):

    old = SchemaNode(Relation(Network, title='network'), title='Old network')
    new = SchemaNode(String(), title='New network (CIDR)')


@view_config(route_name='location_replacenetwork')
class ReplaceNetwork(gocept.directory2.admin.form.Edit):

    title = 'Replace Network'
    schema = ReplaceSchema()
    next_route = 'location_networks'
    buttons = ('replace',)

    def replace_success(self, appstruct):
        self.appstruct = appstruct
        new = Network(network=appstruct['new'])
        new.replace(appstruct['old'])
        # persist must happen after replace, else new doesn't have a vlan
        new.persist()
        return HTTPFound(location=self.request.route_url(
            self.next_route, id=self.context.id))

    def appstruct(self):
        return {}
