from ..db import IDatabase
from ..resourcegroup import ResourceGroup
from ..traffic import Traffic
from .resourcegroup import ResourceGroupTabs
from .view import View, Action
from pyramid.response import Response
from pyramid.view import view_config
import datetime
import json
import zope.component
import dateutil.relativedelta


@view_config(route_name='traffic_by_customer',
             renderer='templates/traffic_by_customer.pt')
class TrafficByCustomer(View):

    title = 'Traffic by customer'

    @property
    def customers(self):
        result = {}
        for resource_group in ResourceGroup.query():
            # XXX modelling customers would make this better
            result.setdefault(resource_group.customer_no, [])
            result[resource_group.customer_no].append(resource_group)
        for customer_no, rgs in sorted(result.items()):
            yield {'customer_no': customer_no, 'rgs': rgs}


class TrafficDetailBase(View):

    def next_month(self):
        next = self._first() + datetime.timedelta(days=31)
        return next.strftime('%Y-%m')

    def previous_month(self):
        next = self._first() - datetime.timedelta(days=1)
        return next.strftime('%Y-%m')

    @property
    def month(self):
        return self.request.params.get(
            'month', datetime.date.today().strftime('%Y-%m'))

    def _first(self):
        return datetime.date(*map(int, tuple(self.month.split('-')) + (1,)))

    def _last(self):
        first = self._first()
        next_month = first + datetime.timedelta(31)
        first_next = next_month.replace(day=1)
        return first_next - datetime.timedelta(days=1)

    @property
    def _resource_groups(self):
        raise NotImplemented

    def _traffic_by_rg(self, group):
        db = zope.component.getUtility(IDatabase)
        return db.query(Traffic).\
            filter(Traffic.resource_group_id == group.id,
                   Traffic.day >= self._first(),
                   Traffic.day <= self._last()).all()

    def total(self):
        result = 0
        for group in self._resource_groups:
            for traffic in self._traffic_by_rg(group):
                result += traffic.traffic
        return format_traffic_from_int(result)


@view_config(route_name='resourcegroup_traffic_detail',
             renderer='templates/traffic_by_resourcegroup_detail.pt')
class TrafficByResourceGroupDetail(TrafficDetailBase):

    @property
    def actions(self):
        yield Action('Previous month', 'icon-backward',
                     'resourcegroup_traffic_detail', id=self.context.id,
                     _query=dict(month=self.previous_month()))
        yield Action('Next month', 'icon-forward',
                     'resourcegroup_traffic_detail', id=self.context.id,
                     _query=dict(month=self.next_month()))

    @property
    def title(self):
        return u'Traffic for resource group "{}"'.format(self.context.name)

    @property
    def subtitle(self):
        return self.month

    @property
    def _resource_groups(self):
        return [self.context]

    @property
    def customer_no(self):
        return self.context.customer_no

    @property
    def traffic_by_ip(self):
        result = {}
        for item in self._traffic_by_rg(self.context):
            if item.ipaddress not in result:
                result[item.ipaddress] = 0
            result[item.ipaddress] += item.traffic
        for ip, traffic in sorted(result.items()):
            yield {'ip': ip, 'traffic': format_traffic_from_int(traffic)}

    @property
    def traffic_by_day(self):
        result = {}
        for item in self._traffic_by_rg(self.context):
            day = item.day.strftime('%Y-%m-%d')
            if day not in result:
                result[day] = 0
            result[day] += item.traffic
        for day, traffic in sorted(result.items()):
            yield {'day': day, 'traffic': format_traffic_from_int(traffic)}


@view_config(route_name='traffic_by_customer_detail',
             renderer='templates/traffic_by_customer_detail.pt')
class TrafficByCustomerDetail(TrafficDetailBase):

    @property
    def actions(self):
        yield Action('Previous month', 'icon-backward',
                     'traffic_by_customer_detail', customer=self.customer_no,
                     _query=dict(month=self.previous_month()))
        yield Action('Next month', 'icon-forward',
                     'traffic_by_customer_detail', customer=self.customer_no,
                     _query=dict(month=self.next_month()))

    @property
    def title(self):
        return u'Traffic for {} in {}'.format(self.customer_no, self.month)

    @property
    def customer_no(self):
        return self.request.matchdict['customer']

    @property
    def _resource_groups(self):
        db = zope.component.getUtility(IDatabase)
        query = db.query(ResourceGroup).\
            filter(ResourceGroup.customer_no == self.customer_no)
        return query.all()

    def by_rg(self):
        for group in self._resource_groups:
            result = 0
            for traffic in self._traffic_by_rg(group):
                result += traffic.traffic
            yield {'resource_group_id': group.id,
                   'resource_group_name': group.name,
                   'traffic': format_traffic_from_int(result)}


def format_traffic_from_int(traffic):
    if not isinstance(traffic, (int, long)):
        return ''
    if traffic >= 1073741824:
        return '%s GiB' % (traffic / 1073741824)
    if traffic >= 1048576:
        return '%s MB' % (traffic / 1048576)
    if traffic >= 1024:
        return '%s kB' % (traffic / 1024)
    if traffic == 1:
        return '%s Byte' % traffic
    return '%s Bytes' % traffic


@view_config(route_name='traffic_chart_data')
def traffic_data(request):
    traffic = Traffic.query().filter(
        Traffic.day == datetime.date(2012, 9, 10)).all()

    rgs = {}
    for record in traffic:
        rgs.setdefault(record.resource_group, 0)
        rgs[record.resource_group] += record.traffic

    result = {'name': 'traffic', 'children': []}
    for rg, traffic in rgs.items():
        result['children'].append(
            dict(name=rg.name, size=traffic))

    return Response(json.dumps(result), content_type='application/json')


@view_config(route_name='traffic_chart',
             renderer='templates/traffic_chart.pt')
class TrafficChart(View):

    title = 'Traffic chart'

    @property
    def customers(self):
        result = {}
        for resource_group in ResourceGroup.query():
            # XXX modelling customers would make this better
            result.setdefault(resource_group.customer_no, [])
            result[resource_group.customer_no].append(resource_group)
        return result


@view_config(route_name='resourcegroup_traffic',
             renderer='templates/traffic_by_month.pt')
class ListTrafficByMonthForRG(ResourceGroupTabs, View):
    """Traffic list on resource group traffic tab."""

    @property
    def title(self):
        return self.context.name

    def _last(self):
        return datetime.date.today()

    def _first(self):
        date = self._last() - dateutil.relativedelta.relativedelta(months=3)
        return datetime.date(date.year, date.month, 1)

    def months(self):
        date = self._first()
        while date <= self._last():
            yield date.strftime('%Y-%m')
            date += dateutil.relativedelta.relativedelta(months=1)

    def traffic(self):
        db = zope.component.getUtility(IDatabase)
        query = db.query(Traffic).filter(
            Traffic.resource_group_id == self.context.id,
            Traffic.day >= self._first(),
            Traffic.day <= self._last()).all()
        result = {}.fromkeys(self.months(), 0)
        for traffic in query:
            result[traffic.day.strftime('%Y-%m')] += traffic.traffic
        for month, traffic in sorted(result.iteritems()):
            yield {'month': month,
                   'traffic': format_traffic_from_int(traffic)}
