from ...location import Location
from ...machine import PhysicalMachine, VirtualMachine
from ...network import Network
from ...resourcegroup import ResourceGroup
from ...traffic import Traffic
import datetime
import gocept.directory2.api.testing
import transaction


class TestRing0Fixture(gocept.directory2.api.testing.XMLRPCTestCase):

    def setUp(self):
        super(TestRing0Fixture, self).setUp()

        self.newyork = newyork = Location.create(abbreviation='ny')

        for vlan in newyork.vlans.values():
            Network.create(
                network='231.20.%s.0/24' % vlan.vlanclass.number,
                vlan=vlan)

            if vlan.name != 'ipmi':
                Network.create(
                    network='2002:470:9aaf:4%s::/64' % vlan.vlanclass.number,
                    vlan=vlan)

        jamaica = Location.create(abbreviation='jamaica')
        for vlan in jamaica.vlans.values():
            if vlan.name in ['ipmi', 'mgm', 'fe', 'srv']:
                Network.create(
                    network='172.20.%s.0/24' % vlan.vlanclass.number,
                    vlan=vlan)
            if vlan.name in ['sto', 'srv', 'fe']:
                Network.create(
                    network='2001:470:9aaf:4%s::/64' % vlan.vlanclass.number,
                    vlan=vlan)

        self.services = services = ResourceGroup.query().filter(
            ResourceGroup.name == 'services').one()
        services.timezone = 'UTC'
        self.stampy = stampy = PhysicalMachine.create(
            name='stampy',
            location=newyork,
            resource_group=services,
            directory_ring=0,
            directory_password='gfhdjk',
            service_description='backup server')

        mgm = stampy.nics['mgm']
        mgm.mac = '00:1e:c9:ad:4a:a6'
        newyork.vlans['mgm'].networks['231.20.1.0/24'].allocate(mgm)

        ipmi = stampy.nics['ipmi']
        ipmi.mac = '00:1e:c9:ad:4a:a7'
        newyork.vlans['ipmi'].networks['231.20.1.0/24'].allocate(ipmi)

        fe = stampy.nics['fe']
        fe.mac = '00:15:17:91:d2:f0'
        newyork.vlans['fe'].networks['231.20.2.0/24'].allocate(fe)

        srv = stampy.nics['srv']
        srv.mac = '00:1e:c9:ad:4a:a0'
        newyork.vlans['srv'].networks['231.20.3.0/24'].allocate(srv)

        sto = stampy.nics['sto']
        sto.mac = '00:1e:c9:ad:4a:a2'
        newyork.vlans['sto'].networks['231.20.4.0/24'].allocate(sto)

        jamaica.vlans['srv'].networks[
            '172.20.3.0/24'].dhcp_dynamic_range = True

        self.test00 = VirtualMachine.create(
            directory_ring=1,
            directory_password=None,
            service_description='test VM',
            name='test00',
            location=newyork,
            resource_group=services,
            memory=512,
            disk=5)

        transaction.commit()

        self.api = self.xmlrpc('/directory')


class TestRing0(TestRing0Fixture):

    def test_check_consistency(self):
        self.assertEqual([], self.api.check_consistency())

    def test_list_virtual_machines(self):
        self.assertEqual([], self.api.list_virtual_machines())

    def test_store_traffic_unknown_ip_is_ignored(self):
        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [('1.1.1.1', 500)])

    def test_store_traffic_adds_up_on_same_day(self):
        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [('231.20.1.2', 500)])
        traffic = Traffic.query().filter(
            Traffic.ipaddress == '231.20.1.2',
            Traffic.day == datetime.date(2012, 6, 13)).one()
        self.assertEqual(500, traffic.traffic)

        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [('231.20.1.2', 300)])
        traffic = Traffic.query().filter(
            Traffic.ipaddress == '231.20.1.2',
            Traffic.day == datetime.date(2012, 6, 13)).one()
        transaction.begin()
        self.assertEqual(800, traffic.traffic)

    def test_store_traffic_reallocated_ips_show_with_correct_history(self):
        rg_test = ResourceGroup.create(name='test')
        krusty = PhysicalMachine.create(
            name='krusty',
            location=self.newyork,
            resource_group=rg_test)

        # 1. Record some traffic for this IP while allocated to stampy in the
        # services RG
        address = self.stampy.nics['mgm'].addresses[0]
        transaction.commit()
        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [(address.address, 12345)])

        # 2. Deallocate the IP and allocate again for a machine in a different
        # RG
        address.nic = None
        address.network.allocate(krusty.nics['mgm'], address)
        transaction.commit()

        # 3. Record some traffic to the IP for the same day
        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [(address.address, 789)])

        # Result: traffic gets recorded to the different RGs correctly
        transaction.begin()
        services_traffic = Traffic.query().filter(
            Traffic.resource_group == self.services,
            Traffic.day == datetime.date(2012, 6, 13),
            Traffic.ipaddress == address.address).one()
        self.assertEquals(12345, services_traffic.traffic)

        test_traffic = Traffic.query().filter(
            Traffic.resource_group == rg_test,
            Traffic.day == datetime.date(2012, 6, 13),
            Traffic.ipaddress == address.address).one()
        self.assertEquals(789, test_traffic.traffic)

    def test_store_traffic_for_deallocated_ips_doesnt_break(self):
        mgm = self.newyork.vlans['mgm']
        addr = mgm.networks['231.20.1.0/24'].addresses['231.20.1.2']
        addr.nic = None
        self.api.store_traffic(datetime.datetime(2012, 6, 13, 13, 38, 0),
                               [(addr.address, 500)])
