from ..location import Location
from ..ipaddress import IPAddress
from ..machine import PhysicalMachine
from ..network import Network
from ..resourcegroup import ResourceGroup
import gocept.directory2.testing
import gocept.testing.assertion
import transaction
import unittest


class AddressTestsBase(unittest.TestCase, gocept.testing.assertion.Ellipsis):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def setUp(self):
        super(AddressTestsBase, self).setUp()

        self.location = Location.create(name='whq')
        self.rg = ResourceGroup(name='rg')

        self.node1 = PhysicalMachine.create(
            name='node1', location=self.location,
            resource_group=self.rg)
        self.nic1 = self.node1.nics['srv']

        self.node2 = PhysicalMachine.create(
            name='node2', location=self.location,
            resource_group=self.rg)
        self.nic2 = self.node2.nics['srv']

        self.network = Network.create(
            network='10.1.1.0/24',
            vlan=self.nic2.vlan)

        transaction.commit()


class NetworkResizingTests(AddressTestsBase):

    def test_assign_larger_net_just_works(self):
        network = self.location.vlans['srv'].networks['10.1.1.0/24']
        network.network = '10.1.1.0/23'

    def test_assign_smaller_net_all_addresses_fit(self):
        network = self.location.vlans['srv'].networks['10.1.1.0/24']
        network.network = '10.1.1.0/25'

    def test_assign_smaller_net_addreses_dont_fit(self):
        net = self.location.vlans['srv'].networks['10.1.1.0/24']
        for i in range(140):
            net.allocate(self.nic1)
        net = self.location.vlans['srv'].networks['10.1.1.0/24']
        self.assertRaises(ValueError, setattr, net, 'network', '10.1.1.0/25')

    def test_assign_different_cidr_with_twin_forbidden(self):
        Network.create(network='10.1.1.0/24', vlan=self.location.vlans['fe'])
        net = self.location.vlans['srv'].networks['10.1.1.0/24']
        self.assertRaises(ValueError, setattr, net, 'network', '10.1.1.0/25')


class AddressAllocationTests(AddressTestsBase):
    # These tests check for the allocation algorithm of IP addresses.
    #
    # In addition to the simple cases this also deals with addresses being
    # allocated by multiple NICs, spread over networks which are shared
    # in multiple VLANs, re-use of addresses once allocated.

    def test_allocate_explicit_address(self):
        address = IPAddress.create(address='10.1.1.1')

        self.network.allocate(self.nic1, address)
        self.assertTrue(address.address in self.network.addresses)
        self.assertEquals(self.nic1, address.nic)

    def test_allocate_explicit_address_wrong_network(self):
        address = IPAddress(address='10.1.2.1')
        self.assertRaises(RuntimeError, self.network.allocate,
                          self.nic1, address)

    def test_allocate_explicit_address_may_not_be_network_address(self):
        address = IPAddress.create(address='10.1.1.0')
        self.assertRaises(RuntimeError, self.network.allocate,
                          self.nic1, address)

    def test_allocate_explicit_address_may_not_be_broadcast_address(self):
        address = IPAddress.create(address='10.1.1.255')
        self.assertRaises(RuntimeError, self.network.allocate,
                          self.nic1, address)

    def test_allocate_automatic_address(self):
        address = self.network.allocate(self.nic1)
        self.assertEquals(address, self.network.addresses[address.address])
        self.assertEquals(self.nic1, address.nic)

    def test_allocate_automatic_address_uses_all_allocatable_addresses(self):
        # Network, router, and broadcast addresses must be avoided.
        # Use a relatively small network so the tests are faster.
        self.network = Network.create(
            network='10.1.1.0/28', vlan=self.location.vlans['srv'])

        addresses = []
        while True:
            try:
                addresses.append(self.network.allocate(self.nic1))
            except RuntimeError:
                break
        self.assertEqual(13, len(addresses))
        self.assertEqual('10.1.1.2', addresses[0].address)
        self.assertEqual('10.1.1.14', addresses[-1].address)

    def test_allocate_automatic_address_reuses_freed_address(self):
        # Network, router, and broadcast addresses must be avoided.
        # Use a relatively small network so the tests are faster.
        self.network = Network.create(
            network='10.1.1.0/28', vlan=self.location.vlans['srv'])

        address = self.network.allocate(self.nic1)
        address.nic = None

        addresses = []
        while True:
            try:
                addresses.append(self.network.allocate(self.nic1))
            except RuntimeError:
                break
        self.assertEqual(13, len(addresses))

    def test_allocate_reused_explicit_address(self):
        address = self.network.allocate(self.nic1)
        address.nic = None

        address2 = self.network.allocate(self.nic2, address)
        self.assertEquals(address2, address)
        self.assert_(address2 is address)

    def test_allocate_used_explicit_address(self):
        address = self.network.allocate(self.nic1)
        self.assertRaises(RuntimeError, self.network.allocate,
                          self.nic2, address)

    def test_allocate_automatic_address_with_identical_twins(self):
        # Let's set up another VLAN with the same network as we already have
        net1 = Network.create(network='10.1.1.0/24',
                              vlan=self.location.vlans['mgm'])
        net2 = self.location.vlans['srv'].networks['10.1.1.0/24']

        addr1 = net1.allocate(self.nic1)
        addr2 = net2.allocate(self.nic2)

        self.assertNotEquals(addr1.address, addr2.address)

    def test_allocate_automatic_address_with_subnet_twins(self):
        # use a smaller network so test runs faster
        net2 = Network.create(network='10.1.1.0/28',
                              vlan=self.location.vlans['srv'])
        # Let's set up another VLAN with a sub-network of one we already have
        net1 = Network.create(network='10.1.1.8/29',
                              vlan=self.location.vlans['mgm'])

        # As the smaller network overlaps the second half of the larger one,
        # we first need to allocate half of the addresses from the larger
        # network.
        for x in range(8):
            net2.allocate(self.nic1)

        addr1 = net1.allocate(self.nic1)
        addr2 = net2.allocate(self.nic2)

        self.assertEquals('10.1.1.10', addr1.address)
        self.assertEquals('10.1.1.11', addr2.address)

        # Following up with more allocations results in the IPs being
        # allocated intertwined between both networks:
        addr3 = net1.allocate(self.nic1)
        addr4 = net2.allocate(self.nic1)

        self.assertEquals('10.1.1.12', addr3.address)
        self.assertEquals('10.1.1.13', addr4.address)

    def test_allocate_explicit_address_used_in_twin(self):
        # Let's set up another VLAN with a sub-network of one we already have
        net1 = Network.create(network='10.1.1.128/25',
                              vlan=self.location.vlans['mgm'])
        net2 = self.location.vlans['srv'].networks['10.1.1.0/24']
        addr = net1.allocate(self.nic1)
        addr2 = IPAddress.create(address=addr.address)
        self.assertRaises(RuntimeError, net2.allocate, self.nic1, addr2)

    def test_alloc_explicit_address_unused_in_twin_removes_it_from_that(self):
        twin = Network.create(network='10.1.1.0/24',
                              vlan=self.location.vlans['fe'])
        address = twin.allocate(self.nic1)
        address.nic = None

        self.network.allocate(self.nic1, address)
        self.assertEqual(self.network, address.network)
        self.assertEqual(address, self.network.addresses[address.address])
        self.assertFalse(address.address in twin.addresses)
        self.assertFalse(address in twin.addresses.values())

    def test_deleting_node_should_release_addresses(self):
        address = self.network.allocate(self.nic1)
        ip_address = address.address
        self.assertTrue(address.in_use, 'address not in use')
        self.node1.delete()
        self.assertFalse(address.in_use, 'address still in use')

        # and we should be able to re-allocate an address object with the same
        # IP but the original object stays untouched.
        address2 = IPAddress.create(address=ip_address)
        self.network.allocate(self.nic2, address2)
        self.assertTrue(address.in_use, 'address not in use')


class NetworkReplaceTests(unittest.TestCase):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def test_after_replace_addresses_are_allocated_in_new_network(self):
        vm = PhysicalMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        location = self.layer['loc0']._refresh()
        old = Network.create(
            vlan=location.vlans['srv'], network='195.62.106.0/27')
        address = IPAddress.create(
            address='195.62.106.14', network=old, nic=vm.nics['srv'])
        transaction.commit()
        new = Network(network='195.62.107.0/27')
        new.replace(old)
        new.persist()
        transaction.commit()
        self.assertEqual('195.62.107.14', vm.nics['srv'].addresses[0].address)
        self.assertEqual(None, address.nic)
        self.assertEqual(location.vlans['srv'], new.vlan)
        self.assertEqual(
            0, Network.query().filter_by(network='195.62.106.0/27').count())
